BEGIN {
    Name        = "Translate Shell"
    Description = "Command-line translator using Google Translate, Bing Translator, Yandex.Translate, etc."
    Version     = "0.9.6.6"
    ReleaseDate = "2017-12-17"
    Command     = "trans"
    EntryPoint  = "translate.awk"
    EntryScript = "translate"
}
